<?php

namespace Drupal\pki_ra\Plugin\rest\resource;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\node\Entity\Node;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\pki_ra\Processors\PKIRACertificateProcessor;
use Drupal\pki_ra\Processors\PKIRACertificationAuthority;
use Drupal\pki_ra\Processors\PKIRARegistrationProcessor;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a resource to get a certificate from a certification authority.
 *
 * @RestResource(
 *   id = "pki_ra_certificate_signing_request_rest_resource",
 *   label = @Translation("Certificate signing request"),
 *   uri_paths = {
 *     "canonical" = "/certificate/signing-request",
 *     "https://www.drupal.org/link-relations/create" = "/certificate/signing-request"
 *   }
 * )
 */
class PKIRACertificateSigningRequestRestResource extends ResourceBase {

  /**
   * Constructs a Drupal\rest\Plugin\ResourceBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   A current user instance.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    AccountProxyInterface $current_user) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('pki_ra'),
      $container->get('current_user')
    );
  }

  /**
   * Responds to POST requests.
   *
   * Handles certificate signing requests (CSRs) by forwarding to a CA, and
   * returning the resulting certificate.
   *
   * @param array $data
   *   * registrationId: The user's registration ID.
   *   * csr: A string representing the CSR itself.
   *   * profile: A string representing the type of certificate requested.
   *
   * @todo Support full PKCS#10 specification: https://tools.ietf.org/html/rfc2986
   */
  public function post($data = []) {

    if (!isset($data['registrationId']) || !$this->requestIsAuthenticated($data['registrationId'])) {
      $this->logger->notice('CSR received, but request failed to authenticate.  Data: %data', ['%data' => serialize($data)]);
      return new ResourceResponse('Cannot find valid security token matching provided registration ID.  Cannot process CSR.', 403);
    }

    if (!isset($data['csr']) || !is_string($data['csr'])) {
      $this->logger->notice('Invalid CSR received with data %data', ['%data' => serialize($data)]);
      return new ResourceResponse('Must be passed as a string within the data array: csr', 400);
    }
    if (!isset($data['profile']) || !is_string($data['profile'])) {
      $this->logger->notice('Invalid CSR received with data %data', ['%data' => serialize($data)]);
      return new ResourceResponse('Must be passed as a string within the data array: profile', 400);
    }

    $this->logger->notice('Valid CSR received from registrant %id with CSR %csr and profile %profile.', [
      '%id' => $data['registrationId'],
      '%csr' => $data['csr'],
      '%profile' => $data['profile'],
    ]);

    $registration_id = $data['registrationId'];
    unset($data['registrationId']);
    return $this->sendRequestToCaAndProcessResults($registration_id, $data);
  }

  /**
   * @todo Replace this with a formal token-based Authentication Provider plug-in.
   */
  protected function requestIsAuthenticated($registration_id) {
    $security_token_key = 'pki_ra_csr_token_' . $registration_id;

    if (!isset($_SESSION[$security_token_key]) ||
        !is_object($registration = Node::load($registration_id)) ||
        ($registration->getType() != PKIRARegistrationProcessor::NODE_TYPE) ||
        !PKIRARegistrationProcessor::isConfirmed($registration) ||
        !isset($registration->get('field_registration_code')->getValue()[0]['value'])) {
      return FALSE;
    }

    if ($_SESSION[$security_token_key] != $registration->get('field_registration_code')->getValue()[0]['value']) {
      return FALSE;
    }

    unset($_SESSION[$security_token_key]);
    $registration_processor = new PKIRARegistrationProcessor($registration);
    $registration_processor->unsetSecurityToken();
    $registration_processor->saveRegistration();
    return TRUE;
  }

  protected function sendRequestToCaAndProcessResults($registration_id, $data) {
    $ca_url = \Drupal::config('pki_ra.settings')->get('certificate_authority_url');
    $ca = new PKIRACertificationAuthority($ca_url);

    $response = $ca->signCertificate($data['csr'], $data['profile'])->getCertificateSigningResponse();
    $status = $ca->getCertificateSigningStatus();
    $headers = $ca->getCertificateSigningHeaders();

    $this->logger->notice('CA at %url responded to CSR with status %status and headers %headers.', [
      '%url' => Xss::filterAdmin($ca_url),
      '%status' => $status,
      '%headers' => serialize($headers),
    ]);

    $certificate_processor = PKIRACertificateProcessor::createCertificate($registration_id, $response['certificate']);
    $certificate_processor->saveCertificate();

    if ($status == 201) {
      $registration_processor = new PKIRARegistrationProcessor(Node::load($registration_id));
      $registration_processor->unsetSecurityToken();
      $registration_processor->saveRegistration();
    }

    return new ResourceResponse($response, $status, $headers);
  }

}
